/*
 * Author:    Konrad Rzeszutek <conradus@iname.com>
 * Date:      Sept 21th, 1999
 * File:      Editor2D/DrawPolygons.java
 *
 * Notes:
 *   This class draws pretty polygons using  DrawLine.
 *   The polygons can either by thick, patterned, both, or none of these.
 *
 */
package Editor2D;

import java.util.*;
import java.awt.*;

public class DrawPolygons  extends  MyPrimitives {
  /*
     * Initializes the DrawPolygons object
     *
     * require:
     *   OffG != null  && canvas != null && OffD != null
     *   ( (state == MyPrimitives.DRAW_NO_PATTERN) || (state == MyPrimitives.DRAW_PATTERN_ON)
     *   || (state == MyPrimitives.DRAW_PATTERN_OFF) || (state == MyPrimitives.DRAW_THICK_ON)
     *   || (state == MyPrimitives.DRAW_THICK_OFF))
     */
    public DrawPolygons (Graphics OffG, Dimension OffD, Canvas canvas, int state) {

	super();
	super.myState = state;
	super.maxPoints = 1;    // The max points we will ever need
	this.canvas = canvas;
	this.OffG = OffG;
	this.OffD = OffD;
	
	vec = new Vector(10,10); 
	vecC = -1;              // Our personal counter for the list of lines.

    }
    
    /*
     * Finds out if more points can be added.
     * 
     * return:
     *  true.
     *
     */
    public boolean canAdd() {
    
	return true;
	
    }

    /*
     * Adds a single point to the array of points.
     *
     * require:
     *   p instanceof Point
     *
     * note:
     *   if (isDeleted) then nothing will happen.
     *   if (!canAdd) then add() will invoke this.draw().
     */
    public void add (Point pt) {
    
	if ((maxPoints == nPoints) && (!deleted)) {
	    // Pass on the drawing state (in case we changed it)
	    ((MyPrimitives)(vec.elementAt(vecC))).setChanges(myState);
	    ((MyPrimitives)(vec.elementAt(vecC))).add(pt); 
	    vecC++;		
	    // Create another line and end this ending edge to it.
	    vec.addElement((Object) (new DrawLine(OffG,OffD,canvas,myState)));
	    ((MyPrimitives)(vec.elementAt(vecC))).add(pt);	    
	} else {
	    // Don't have any points, hence we nned to create one
	    vecC++;		
	    vec.addElement((Object) (new DrawLine(OffG,OffD,canvas,myState)));
	    ((MyPrimitives)(vec.elementAt(vecC))).add(pt);
	    nPoints++; // Now we will just add new points.
	}
    }
    /*
     * Finds the distance between points in this object
     * and the given point.
     *
     * require:
     *  pt instanceof Point
     *
     * return
     *  the distance from the perpendicular point to the point.
     *
     */
    public int distance(Point pt) {
    
	int myDistance = 0;
	int currentD = 0;
	int index = 0;
	int i;           // Simple counter
	
	if (!deleted) {
	    if (vecC >= 0) {    // If we have enough points
		myDistance = currentD = ((MyPrimitives)(vec.elementAt(0))).distance(pt);
		// Returns the smallest distance from all the faces.
		for (i=0; i<= vecC;i++) {
		    currentD = ((MyPrimitives)(vec.elementAt(i))).distance(pt);
		    if ( currentD <= myDistance) {
			myDistance =  currentD;index = i;
		    }
		}
	    }
	} else 
	    myDistance = MyPrimitives.POS_INFINITY;
	
	return myDistance;
    }

    
    /*
     * Changes the way the object is being drawn.
     *
     * require:
     *  (state == MyPrimitives.DRAW_NO_PATTERN) || (state == MyPrimitives.DRAW_PATTERN_ON)
     *   || (state == MyPrimitives.DRAW_PATTERN_OFF) || (state == MyPrimitives.DRAW_THICK_ON)
     *   || (state == MyPrimitives.DRAW_THICK_OFF))
     *
     */
     public void setChanges(int state) {
     
	this.myState = state;
	if (vecC >= 0) {
	    ((MyPrimitives)(vec.elementAt(vecC))).setChanges(state);
	    myState = ((MyPrimitives)(vec.elementAt(vecC))).getState();
	}
     }

    /* 
     * Draws the object.
     */
    public void draw () {
	
	int i; // Simple counter
	
	if ((maxPoints == nPoints) && (!deleted)) {
	    for (i=0;i<= vecC;i++)
		((MyPrimitives)(vec.elementAt(i))).draw();
	}
    }

    /*
     * Returns a representation of the objects actions and its points
     *
     * return:
     *  dump() instanceof String.
     */    
    public String dump() {
    
	String msg = "\nOperation: polygon";
        int i;
	// Go thru each face and get its points string representation.
	if (deleted) msg+=" [DELETED]";
	
	if (maxPoints == nPoints) {
	    for (i=0;i<= vecC;i++)
		msg += ((MyPrimitives)(vec.elementAt(i))).toString();
	}
	return msg;
    }
    
    /*
     * Returns true if this is a polygon.
     * (Used for DrawFill.java)
     *
     * return:
     *  true
     */
    public boolean isPolygon() {

	return true;

    }

    /*
     * Gets the maximum current amount of edges(points)
     * that are contained within this object.
     *
     * return:
     *   0...n
     */
    public int getEdgeLength() {
	
	return vec.size();
    }

    /*
     * Returns a specific Point object for the requested edge.
     * 
     */
    public Point getEdge(int index) {

	if (index <= vec.size())
	    return ( ((MyPrimitives)vec.elementAt(index)).getEdge(0));
	else
	    return null;

    }
    private Graphics OffG;  // The place where we will draw
    private Dimension OffD; // Used to find the x,y origin in DrawLine
    private Canvas canvas;  // To get the canvas background color.
    private int vecC;       // Counter for the lines we have
    private Vector vec;     // The list of lines.
}
