/*
 * Author: Konrad Rzeszutek <conradsu@iname.com>
 * Date:   Wed, 22th 1999
 * File:   Editor2D/MyList.java
 *
 * Notes:
 *
 *  Serves the drawing classes with a list.
 *
 */
package Editor2D;

import java.util.*;


public class MyList extends Vector {

    // Constants
    static final int CAPACITY = 100;
    static final int GROWS    = 10;

    /*
     * Initializes the Object
     */
    public MyList () {

	super(CAPACITY,GROWS);
	vecC = -1;
    }

    /*
     * Adds a atomoton to the list of MyPrimitives
     *
     * require:
     *   obj instanceof MyPrimitivies 
     *
     */
    public void addItem(MyPrimitives obj) {	

	vecC++;		
	this.addElement((Object)obj);
    }

    /*
     * Returns the requested object at the given index.
     *
     * require:
     *   index >= 0
     * return:
     *   itemAt() instanceof MyPrimitivies || itemAt()==null (if length() < 0)
     */
    public MyPrimitives itemAt(int index) {

	if (index <= vecC)
	    return (MyPrimitives)this.elementAt(index);
	else return null;
    }
    /*
     * Returns the last item of the list.
     *
     * return
     * return:
     *   itemAt() instanceof MyPrimitivies || itemAt()==null (if length() < 0)
     */
    public MyPrimitives lastItem () {

	if (vecC >= 0)
	    return (MyPrimitives)this.elementAt(vecC);
	else return null;
    }
    /*
     * Returns the "counter" of the objects deposited.
     * Starts at  0, not 1. (ie, if one element is in, length() will give 0)
     *
     * return:
     *  -1 (if no items)
     *  0..n (for n+1 items)
     */
    public int length() {

	return vecC;
    }

    /*
     * Delete the MyPrimitivie at the specified index
     *
     * require:
     *  index >= 0
     *
     */
    public void delElementAt(int index) {

	if (index <= vecC) { 
	    this.removeElementAt(index);
	    vecC --;
	}
    }

    /*
     * Clear the whole list
     *
     */
    public void delAllElements() {

	this.removeAllElements();
	vecC = -1;
    }

    private int vecC; // Counter for items.
}
